package ru.dbotthepony.mc.otm.block.entity.tech

import net.minecraft.core.BlockPos
import net.minecraft.server.level.ServerLevel
import net.minecraft.server.level.ServerPlayer
import net.minecraft.world.entity.ExperienceOrb
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.block.state.BlockState
import ru.dbotthepony.mc.otm.block.entity.MatteryWorkerBlockEntity
import ru.dbotthepony.mc.otm.capability.energy.WorkerEnergyStorage
import ru.dbotthepony.mc.otm.config.MachinesConfig
import ru.dbotthepony.mc.otm.container.MatteryContainer
import ru.dbotthepony.mc.otm.container.HandlerFilter
import ru.dbotthepony.mc.otm.core.math.Decimal
import ru.dbotthepony.mc.otm.menu.tech.PlatePressMenu
import ru.dbotthepony.mc.otm.registry.MBlockEntities
import ru.dbotthepony.mc.otm.registry.MRecipes

class PlatePressBlockEntity(
	p_155229_: BlockPos,
	p_155230_: BlockState
) : MatteryWorkerBlockEntity<MatteryWorkerBlockEntity.ItemJob>(MBlockEntities.PLATE_PRESS, p_155229_, p_155230_, ::ItemJob) {
	val energy = WorkerEnergyStorage(this::setChangedLight, MachinesConfig.PLATE_PRESS)
	val inputContainer = MatteryContainer(this::itemContainerUpdated, 1).also(::addDroppableContainer)
	val outputContainer = MatteryContainer(this::itemContainerUpdated, 1).also(::addDroppableContainer)

	var experience = 0.0

	fun popExperience(player: ServerPlayer) {
		val whole = experience.toInt()

		if (whole > 0) {
			experience -= whole
			ExperienceOrb.award(player.level as ServerLevel, player.position(), whole)
		}
	}

	val energyConfig = ConfigurableEnergy(energy)
	val itemConfig = ConfigurableItemHandler(
		input = inputContainer.handler(HandlerFilter.OnlyIn),
		output = outputContainer.handler(HandlerFilter.OnlyOut),
	)

	init {
		savetable(::energy, ENERGY_KEY)
		savetable(::inputContainer)
		savetable(::outputContainer)
		savetables.double(::experience)
	}

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu {
		return PlatePressMenu(containerID, inventory, this)
	}

	override fun onJobFinish(job: ItemJob): Status {
		if (job.itemStack.isEmpty)
			return Status.SUCCESS

		if (!outputContainer.fullyAddItem(job.itemStack))
			return Status.FAILURE_ITEM

		experience = (experience + job.experience).coerceAtMost(100.0)
		return Status.SUCCESS
	}

	override fun computeNextJob(): Pair<ItemJob?, IdleReason?> {
		if (energy.batteryLevel.isZero) {
			return null to IdleReason.POWER
		}

		val recipe = level?.recipeManager?.getRecipeFor(MRecipes.PLATE_PRESS, inputContainer, level!!)?.orElse(null) ?: return null to IdleReason.ITEM
		inputContainer[0].shrink(1)
		inputContainer.setChanged(0)
		return ItemJob(recipe.resultItem, recipe.workTime.toDouble(), BASELINE_CONSUMPTION, experience = recipe.experience.sample(level!!.random)) to null
	}

	companion object {
		private val BASELINE_CONSUMPTION = Decimal(15)
	}
}
